--author: Andy_V (AVSR Modding)
--v1.1
--Special for Zil Expansion Package
--
--Help Tip on first enter/attach
--Turn Lights nodes visibility
--Fridge Sound system
--Tip Engine system
--Randomly Numbers
--Entering Moving Nodes

commonFunctions = {}
helps = {}

function commonFunctions.prerequisitesPresent(specializations)
    return true;
end;

function commonFunctions:load(savegame)
	self.helpType = getXMLString(self.xmlFile, "vehicle.helpTip.tip#tipType");
	
	if savegame ~= nil then
		local tipHelpStateLoad = getXMLBool(savegame.xmlFile, savegame.key.."#helpState");
	 	if tipHelpStateLoad ~= nil then
			helps[self.helpType] = tipHelpStateLoad;
		end;
	end;
	
	if hasXMLProperty(self.xmlFile, "vehicle.offNodes")	then	
		self.offNodes = true;
		self.leftNode = {};
		self.rightNode = {};
		self.hazardNode = {};
		local o = 0;
		while true do
			local leftNumber = string.format("vehicle.offNodes.leftTurn#index%d", o);
			local leftIndex = getXMLString(self.xmlFile, leftNumber);
			if not leftIndex then
				break;
			end;
			if leftIndex then
				self.leftNode[o] = Utils.indexToObject(self.components, leftIndex);
			end;
			o = o + 1;
		end;
		local o = 0;
		while true do
			local rightNumber = string.format("vehicle.offNodes.rightTurn#index%d", o);
			local rightIndex = getXMLString(self.xmlFile, rightNumber);
			if not rightIndex then
				break;
			end;
			if rightIndex then
				self.rightNode[o] = Utils.indexToObject(self.components, rightIndex);
			end;
			o = o + 1;
		end;
		local o = 0;
		while true do
			local hazardNumber = string.format("vehicle.offNodes.hazardLights#index%d", o);
			local hazardIndex = getXMLString(self.xmlFile, hazardNumber);
			if not hazardIndex then
				break;
			end;
			if hazardIndex then
				self.hazardNode[o] = Utils.indexToObject(self.components, hazardIndex);
			end;
			o = o + 1;
		end;
	end;
	
	self.fridgeSystem = {};
	local fridgeXml = "vehicle.fridgeSystem";
	self.fridgeSystem.exist = getXMLBool(self.xmlFile, fridgeXml.."#fridgeEnabled");
	if self.fridgeSystem.exist == true then
		self.fridgeSystem.work = SoundUtil.loadSample(self.xmlFile, {}, fridgeXml..".fridgeSound", nil, self.baseDirectory, self.components[1].node);
		self.fridgeSystem.percent = getXMLInt(self.xmlFile, fridgeXml.."#percentToEnable");
		local fillTypes = getXMLString(self.xmlFile, fridgeXml..".coldGoods#fillTypes");
		self.coldGoogs = {};
		if fillTypes then
			fillTypes = Utils.splitString(" ", fillTypes);
			for z=1, #fillTypes do
				local fillType = FillUtil.fillTypeNameToInt[fillTypes[z]];
				if fillType then
					self.coldGoogs[fillType] = true;
				end;
			end;
		end;
	end;
	
	self.tipEngineSystem = {};
	local tipEngineXml = "vehicle.tipEngineSystem";
	self.tipEngineSystem.exist = getXMLBool(self.xmlFile, tipEngineXml.."#tipEngineEnabled");
	if self.tipEngineSystem.exist == true then
		self.tipEngineSystem.work = SoundUtil.loadSample(self.xmlFile, {}, tipEngineXml..".engineSound", nil, self.baseDirectory, self.components[1].node);
	end;
	
	if hasXMLProperty(self.xmlFile, "vehicle.randomlyNumbers") then
	    self.randomlyNumbers = {};
		self.randomlyNumbersActive = true;
		if savegame ~= nil then
		    local numberLoad = getXMLInt(savegame.xmlFile, savegame.key.."#randomNumber");
	 	    if numberLoad ~= nil then
			    self.randomlyNumbers.number = numberLoad;
				self.randomlyNumbers.random = true;
			else
			    self.randomlyNumbers.number = 0;
			end;
		elseif savegame ~= nil and savegame.resetVehicles then
			self.randomlyNumbers.random = true;
	    elseif self.randomlyNumbers.random ~= true then
		    local randomMin = getXMLInt(self.xmlFile, "vehicle.randomlyNumbers.randomizer#minValue");
			local randomMax = getXMLInt(self.xmlFile, "vehicle.randomlyNumbers.randomizer#maxValue");
			local number = math.random(randomMin,randomMax);
			self.randomlyNumbers.number = number;
			self.randomlyNumbers.random = true;
		end;
		
	    local i = 0;
        while true do
		    local numberIndex = string.format("vehicle.randomlyNumbers.randomlyNumber(%d)", i);
			local numberNode = getXMLString(self.xmlFile, numberIndex.."#index");
		    if not numberNode then
		        break;
		    end;
			self.randomlyNumbers.index = Utils.indexToObject(self.components, numberNode);
		    Utils.setNumberShaderByValue(self.randomlyNumbers.index, math.floor(self.randomlyNumbers.number), 0);
		    i = i + 1;
	    end;
	end;
	
	if hasXMLProperty(self.xmlFile, "vehicle.movingNodes") then
	    self.movingNodes = {};
		local x = 0;
        while true do
		    local movingNodeNumber = string.format("vehicle.movingNodes.movingNode(%d)", x);
			local movingNodeIndex = getXMLString(self.xmlFile, movingNodeNumber.."#index");
		    if not movingNodeIndex then
		        break;
		    end;
			self.movingNodes.index = Utils.indexToObject(self.components, movingNodeIndex);
			self.movingNodes.buyTrans = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, movingNodeNumber.."#buyTrans"), 3), {0, 0, 0});
			if self.movingNodes.buyTrans then
			    setTranslation(self.movingNodes.index, self.movingNodes.buyTrans[1], self.movingNodes.buyTrans[2], self.movingNodes.buyTrans[3]);
		    end;
			x = x + 1;
	    end;
	end;
end;

function commonFunctions:getSaveAttributesAndNodes(nodeIdent)								
	if self.helpType ~= nil then
	    local attributes = 'helpState="'
	    local stateHelpTip = helps[self.helpType]
	    attributes = attributes..tostring(stateHelpTip)..""
	    attributes = attributes..'"'
	    return attributes
	end;
	
	if self.randomlyNumbersActive == true then
	    local attributes = 'randomNumber="'
	    local savingNumber = self.randomlyNumbers.number
	    attributes = attributes..tostring(savingNumber)..""
	    attributes = attributes..'"'
	    return attributes
	end;
end;

function commonFunctions:onEnter(isControlling)
	if isControlling ~= nil and self.helpType ~= nil then
	    if helps[self.helpType] ~= true then
			g_currentMission.inGameMessage:showMessage(g_i18n:getText("zilPackTipMain"), g_i18n:getText(self.helpType), -1, nil, nil, nil);
			helps[self.helpType] = true;
		end;
	end;
end;

function commonFunctions:onLeave()
end;

function commonFunctions:onAttach(attacherVehicle, jointDescIndex)
	if self.tipHelpState ~= true and self.helpType ~= nil then
		g_currentMission.inGameMessage:showMessage(g_i18n:getText("zilPackTipMain"), g_i18n:getText(self.helpType), -1, nil, nil, nil);
		self.tipHelpState = true;
    end;
end;

function commonFunctions:onDetach(attacherVehicle, jointDescIndex)
end;

function commonFunctions:delete()
    if self.isClient then
        SoundUtil.deleteSample(self.fridgeSystem.work);
		SoundUtil.deleteSample(self.tipEngineSystem.work);
	end;
end;

function commonFunctions:mouseEvent(posX, posY, isDown, isUp, button)
end;

function commonFunctions:keyEvent(unicode, sym, modifier, isDown)
end;

function commonFunctions:update(dt)
	if self.isClient and self.offNodes and self.leftNode[0] then
		for o1=0, #self.leftNode do
			if self.turnLightState == Lights.TURNLIGHT_LEFT or self.turnLightState == Lights.TURNLIGHT_HAZARD then
				setVisibility(self.leftNode[o1], false);
			else
				setVisibility(self.leftNode[o1], true);
			end;
		end;
	end;
	if self.isClient and self.offNodes and self.rightNode[0] then
		for o2=0, #self.rightNode do
			if self.turnLightState == Lights.TURNLIGHT_RIGHT or self.turnLightState == Lights.TURNLIGHT_HAZARD then
				setVisibility(self.rightNode[o2], false);
			else
				setVisibility(self.rightNode[o2], true);
			end;
		end;
	end;
	if self.isClient and self.offNodes and self.hazardNode[0] then
		for o3=0, #self.hazardNode do
			if self.turnLightState == Lights.TURNLIGHT_HAZARD then
				setVisibility(self.hazardNode[o3], false);
			else
				setVisibility(self.hazardNode[o3], true);
			end;
		end;
	end;
	
	if self.isClient and self.fridgeSystem.exist == true then
        local percent = self:getFillLevel() / self:getCapacity();
		local fillType = self.fillUnits[1].currentFillType;
		if percent*100 > self.fridgeSystem.percent and not self:getIsUnfolded() and self.tipState ~= Trailer.TIPSTATE_OPENING and self.tipState ~= Trailer.TIPSTATE_OPEN and self.tipState ~= Trailer.TIPSTATE_CLOSING and self.coldGoogs[fillType] then
			if self.fridgeSystem.work ~= nil and self:getIsActiveForSound() then
				SoundUtil.play3DSample(self.fridgeSystem.work);
		    end;
		else
		    SoundUtil.stop3DSample(self.fridgeSystem.work);
	    end;
	end;
	
	if self.isClient and self.tipEngineSystem.exist == true then
		if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_CLOSING or self.tipState == Trailer.TIPSTATE_OPEN then
			if self.tipEngineSystem.work ~= nil and self:getIsActiveForSound() then
				SoundUtil.play3DSample(self.tipEngineSystem.work);
		    end;
		else
		    SoundUtil.stop3DSample(self.tipEngineSystem.work);
	    end;
	end;
end;

function commonFunctions:updateTick(dt)
end;

function commonFunctions:draw()	
end;
